from llama_index.embeddings.upstage import UpstageEmbedding
from llama_index.core import VectorStoreIndex, Document
from typing import List, Dict
import logging
import json


def load_json_file(json_path: str):
    logging.info(f"JSON 파일 로드: {json_path}")
    try:
        with open(json_path, 'r', encoding='utf-8') as f:
            articles = json.load(f)
    except FileNotFoundError:
        logging.error(f"JSON 파일을 찾을 수 없습니다: {json_path}")
        return
    except json.JSONDecodeError as e:
        logging.error(f"JSON 파일 파싱 오류: {e}")
        return
    except Exception as e:
        logging.error(f"JSON 파일을 읽는 중 오류 발생: {e}")
        return
    
    if not isinstance(articles, list):
        logging.error("JSON 파일의 형식이 올바르지 않습니다. 기사 목록이 필요합니다.")
        return
    return articles


def create_documents_from_json(articles: List[Dict]) -> List[Document]:
    """
    JSON 데이터를 기반으로 Document 객체를 생성합니다.
    """
    documents = []
    for article in articles:
        try:
            # JSON 구조에서 text와 metadata 추출
            doc_text = article.get("text", "No Content")
            metadata = article.get("metadata", {})
            
            # Document 생성
            doc = Document(
                text=doc_text,
                metadata=metadata
            )
            documents.append(doc)
        except Exception as e:
            logging.error(f"Document 생성 중 오류 발생: {e}")
    
    logging.info(f"문서 생성 완료: {len(documents)}개")
    return documents


def create_and_save_index(documents: List[Document], index_file: str, embeddings):
    """
    Document 리스트를 기반으로 인덱스를 생성하고 저장합니다.
    """
    logging.info("인덱스 생성 시작")
    index = VectorStoreIndex.from_documents(documents, embed_model=embeddings)
    index.storage_context.persist(persist_dir=index_file)
    logging.info(f"인덱스가 저장되었습니다: {index_file}")
    return index


def build_index_from_json(json_path: str, index_file: str, embeddings):
    """
    JSON 파일을 로드하고, Document를 생성한 뒤 인덱스를 생성 및 저장합니다.
    """
    # 1. JSON 파일 로드
    articles = load_json_file(json_path)
    if not articles:
        return
    
    # 2. Document 생성
    documents = create_documents_from_json(articles)
    
    # 3. 인덱스 생성 및 저장
    return create_and_save_index(documents, index_file, embeddings)