import pandas as pd
import logging
import logging.config
import yaml
import os
from apps.airflow_pipeline.dags.indexer import build_index_from_json
import json
from typing import List, Dict
from llama_index.core import StorageContext, load_index_from_storage, Document
from llama_index.embeddings.upstage import UpstageEmbedding
from apps.airflow_pipeline.dags.src.feed_loader import FeedLoader


def initialize_environment(config):
    environment = config.get("environment", "dev")
    logging.info(f"Initializing environment: {environment}")

    if environment == "dev":
        from dotenv import load_dotenv
        load_dotenv()
        api_key = os.getenv("UPSTAGE_API_KEY")
        if not api_key:
            logging.warning("API key not found in environment variables.")
        return api_key
    elif environment == "prod":
        logging.info("Production environment does not use API key loading.")
        return None
    
def load_config(config_path='configs/config.yaml'):
    with open(config_path, 'r', encoding='utf-8') as file:
        return yaml.safe_load(file)
    
def setup_logging(config):
    logging.config.dictConfig(config.get("logging", {}))
    logging.info("Logging has been configured.")

def save_articles_to_json(articles: List[Dict], json_path: str) -> None:
    logging.info(f"Saving {len(articles)} articles to JSON file: {json_path}")
    try:
        with open(json_path, 'w', encoding='utf-8') as f:
            json.dump(articles, f, ensure_ascii=False, indent=4)
        logging.info("JSON 파일 저장 완료.")
    except Exception as e:
        logging.error(f"JSON 파일 저장 s중 오류 발생: {e}")


def initialize_feed_processing(feed_specs, article_file, index_folder, embeddings):
    """
    RSS 피드 처리 파이프라인 초기화.

    - `article_file`이 없으면 RSS 피드를 파싱하고 JSON 파일로 저장.
    - `index_file`이 없으면 JSON 데이터를 기반으로 인덱스를 생성하고 저장.
    - 둘 다 있으면 저장된 인덱스를 로드.
    """
    if not os.path.exists(article_file):
        # 1. article_file이 없으면 RSS 피드 파싱 후 저장
        logging.info("Article file doesn't exist. Parsing RSS feed and saving as JSON.")
        loader = FeedLoader(feed_specs)
        save_articles_to_json(loader.parse_rss_feed(), article_file)

    if not os.path.exists(index_folder):
        # 2. index_file이 없으면 JSON 파일로부터 인덱스 생성
        logging.info("Index file doesn't exist. Creating index from JSON articles.")
        index = build_index_from_json(article_file, index_folder, embeddings=embeddings)
    else:
        # 3. index_file이 존재하면 저장된 인덱스 로드
        logging.info("Index file exists. Loading index from storage.")
        storage_context = StorageContext.from_defaults(persist_dir=index_folder)
        index = load_index_from_storage(storage_context, embed_model=embeddings)

    return index

def main():
    config = load_config()
    setup_logging(config)
    logger = logging.getLogger("PressianProcessor")
    logger.info("Logging with combined configuration.")
    api_key = initialize_environment(config)


    feed_specs = config['app']['data']['feed_specs']
    article_file = config['app']['data']['articles']
    index_folder = config['app']['data']['index_folder']
    model = config['app']['embedding']['model_name']


    embedding = UpstageEmbedding(
            api_key = api_key,
            model = model
        )

    index = initialize_feed_processing(feed_specs, article_file, index_folder, embedding)

    query_engine =  index.as_query_engine()

    print("ready!")
    while True:
        response = query_engine.query(input().rstrip())
        print(response)

if __name__ == "__main__":
    main()
